from django.db import models
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _

from ..database_supervisor.models import Schedule, Task
from ..database_auth.models import Device


class Report(models.Model):
    user = models.ForeignKey(User)
    device = models.ForeignKey(Device)
    created = models.DateTimeField(auto_now_add=True)
    created_ip = models.GenericIPAddressField()
    task_id = models.IntegerField() # models.ForeignKey(Task)
    app_version = models.TextField()
    report_time = models.DateTimeField()


class LocalInformation(models.Model):
    available_disk_space = models.BigIntegerField(null=True)
    battery_level = models.SmallIntegerField(null=True)
    connection_mode = models.SmallIntegerField(null=True)
    cpu_usage = models.FloatField(null=True)
    free_memory = models.IntegerField(null=True)
    signal_strength = models.SmallIntegerField(null=True)
    tbm_active = models.BooleanField(default=False)
    available_traffic = models.IntegerField(null=True, default=0)
    available_mobile_traffic = models.IntegerField(null=True, default=0)
    used_traffic = models.IntegerField(null=True, default=0)
    used_mobile_traffic = models.IntegerField(null=True, default=0)

    class Meta:
        abstract = False
        verbose_name = "Local Information"


class BaseResult(models.Model):
    report = models.ForeignKey("Report")

    start_time = models.DateTimeField(null=True)
    end_time = models.DateTimeField(null=True)
    duration = models.IntegerField(null=True)
    measure_uuid = models.TextField(null=True)
    conflicting_tasks = models.TextField(null=True)
    cross_traffic = models.TextField(null=True)
    pre_info = models.ForeignKey("LocalInformation", related_name="%(app_label)s_%(class)s_pre")
    post_info = models.ForeignKey("LocalInformation", related_name="%(app_label)s_%(class)s_post")
    error = models.TextField(null=True)

    class Meta:
        abstract = True
        verbose_name = "Base Result"


class PingResult(BaseResult):
    round_trip_ms = models.CommaSeparatedIntegerField(max_length=1024,null=True) # TODO these are floats, but works for now
    round_trip_min = models.FloatField(null=True)
    round_trip_max = models.FloatField(null=True)
    round_trip_avg = models.FloatField(null=True)
    round_trip_stdev = models.FloatField(null=True)
    round_trip_count = models.IntegerField(null=True) # not relevant after 2.4.0
    round_trip_sent = models.IntegerField(null=True, default=0)
    round_trip_received = models.IntegerField(null=True, default=0)

    class Meta:
        abstract = False
        verbose_name = "Ping Result"


class BTCResult(BaseResult):
    values = models.CommaSeparatedIntegerField(max_length=1024, null=True)
    min = models.IntegerField(null=True)
    avg = models.IntegerField(null=True)
    max = models.IntegerField(null=True)
    stdev = models.IntegerField(null=True)

    class Meta:
        abstract = False
        verbose_name = "BTC Result"


class HTTPDownloadResult(BaseResult):
    actual_num_threads = models.SmallIntegerField(null=True)
    results_ok = models.BooleanField(default=False)
    bandwidth_bps_avg = models.FloatField(null=True, default=None)

    class Meta:
        abstract = False
        verbose_name = "HTTP Download Result"

class HTTPThreadResult(models.Model):
    http_download = models.ForeignKey("HttpDownloadResult")

    min = models.FloatField(null=True)
    avg = models.FloatField(null=True)
    max = models.FloatField(null=True)
    avg = models.FloatField(null=True)
    slots = models.CommaSeparatedIntegerField(max_length=1024, null=True)

    class Meta:
        abstract = False
        verbose_name = "HTTP Thread Result"

class UPnPResult(BaseResult):
    data = models.TextField()

    class Meta:
        abstract = False
        verbose_name = "UPnP Result"


class PacketTrainsResult(BaseResult):
    sending_speed = models.IntegerField(null=True)
    receiving_speed = models.IntegerField(null=True)

    class Meta:
        abstract = False
        verbose_name = "Packet Trains Result"


class DnsLookupResult(BaseResult):
    record_id = models.CharField(max_length=64,null=True)
    name = models.TextField(null=True)
    ttl = models.IntegerField(null=True)
    value = models.TextField(null=True)
    type = models.TextField(null=True)

    class Meta:
        abstract = False
        verbose_name = "DNS Lookup Result"


class ReverseDnsLookupResult(BaseResult):
    hostname = models.TextField(null=True)
    address = models.TextField(null=True)

    class Meta:
        abstract = False
        verbose_name = "Reverse DNS Lookup Result"


class TracerouteResult(BaseResult):
    hop_count = models.SmallIntegerField(null=True)

    class Meta:
        abstract = False
        verbose_name = "Traceroute Result"


class TracerouteHopResult(models.Model):
    traceroute = models.ForeignKey("TracerouteResult")

    ip = models.GenericIPAddressField(null=True)
    ttl = models.IntegerField(null=True)
    rtt_min = models.FloatField(null=True)
    rtt_max = models.FloatField(null=True)
    rtt_avg = models.FloatField(null=True)
    rtt_stdev = models.FloatField(null=True)
    rtt_count = models.IntegerField(null=True)

    class Meta:
        abstract = False
        verbose_name = "Traceroute Hop Result"


class TracerouteHopPingResult(models.Model):
    hop = models.ForeignKey("TracerouteHopResult")

    rtt_us = models.IntegerField(null=True)
    response = models.IntegerField(null=True)

    class Meta:
        abstract = False
        verbose_name = "Traceroute Hop Ping Result"


class TracerouteReasoningPaths(models.Model):
    created = models.DateTimeField(auto_now_add=True)
    pathId = models.TextField()
    report = models.ForeignKey(Report)


class TracerouteReasoningTasks(models.Model):
    name = models.CharField(max_length=255)
    description = models.TextField(blank=True)

    created = models.DateTimeField(auto_now_add=True)
    created_by = models.ForeignKey(User, verbose_name=_("User"))
    task = models.ForeignKey(Task)
    active = models.BooleanField(default=True)
    schedules = models.CommaSeparatedIntegerField(blank=True, null=True, max_length=10000)