from django.db import models
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _

from ..database_auth.models import Device


class Base(models.Model):
    name = models.CharField(max_length=255)
    description = models.TextField(blank=True)
    
    created = models.DateTimeField(auto_now_add=True)
    created_by = models.ForeignKey(User, verbose_name=_("User"))
    
    modified = models.DateTimeField(auto_now=True)

    def __str__(self):
        return self.name

    class Meta:
        abstract = True


class TaskOption(Base):
    class Meta:
        abstract = False
        verbose_name = "Task Option"

    def __str__(self):
        return "%s: %s" % (self.__class__.__name__, self.name)


class TaskOptionPing(TaskOption):
    type = models.CharField(max_length=20, choices=(("System", "System"), ("Tcp", "Tcp"), ("Udp", "Udp")))
    host = models.CharField(max_length=128)
    count = models.IntegerField(default=4)
    timeout = models.IntegerField(default=1000)
    interval = models.IntegerField(default=200)
    ttl = models.IntegerField(default=64)
    destination_port = models.IntegerField(default=33434)
    source_port = models.IntegerField(default=33434)
    payload = models.IntegerField(default=74)

    class Meta:
        abstract = False
        verbose_name = "Task Option: Ping"


class TaskOptionBTC(TaskOption):
    type = models.CharField(max_length=100, choices=(("up", "upstream"), ("down", "downstream")))
    host = models.CharField(max_length=64)
    port = models.IntegerField(default=5105)
    initial_data_size = models.IntegerField(default=1024*1024)
    slices = models.IntegerField(default=10)

    class Meta:
        abstract = False
        verbose_name = "Task Option: BTC"


class TaskOptionDnsLookup(TaskOption):
    host = models.CharField(max_length=128)
    dns_server = models.CharField(max_length=128, blank=True)

    class Meta:
        abstract = False
        verbose_name = "Task Option: DNS Lookup"


class TaskOptionHttpDownload(TaskOption):
    url = models.CharField(max_length=256)
    avoid_caches = models.BooleanField(default=False)
    threads = models.IntegerField(default=1)
    target_time = models.IntegerField(default=10000)
    ramp_up_time = models.IntegerField(default=3000)
    slot_length = models.IntegerField(default=1000)

    class Meta:
        abstract = False
        verbose_name = "Task Option: HTTP Download"


class TaskOptionPacketTrains(TaskOption):
    host = models.CharField(max_length=128)
    port = models.IntegerField(default=5105)
    packet_size = models.IntegerField(default=1000)
    train_length = models.IntegerField(default=48)
    iterations = models.IntegerField(default=1)
    rate_min = models.BigIntegerField(default=10485760)
    rate_max = models.BigIntegerField(default=262144000)
    delay = models.BigIntegerField(default=200000000)

    class Meta:
        abstract = False
        verbose_name = "Task Option: Packet Train"


class TaskOptionReverseDnsLookup(TaskOption):
    ip = models.CharField(max_length=128)

    class Meta:
        abstract = False
        verbose_name = "Task Option: Reverse DNS Lookup"


class TaskOptionTraceroute(TaskOption):
    host = models.CharField(max_length=128)
    count = models.IntegerField(default=3)
    interval = models.IntegerField(default=1000)
    receive_timeout = models.IntegerField(default=1000)
    destination_port = models.IntegerField(default=33434)
    source_port = models.IntegerField(default=33434)
    payload = models.IntegerField(default=74)
    ping_type = models.CharField(max_length=20, choices=(("System", "System"), ("Tcp", "Tcp"), ("Udp", "Udp")))

    class Meta:
        abstract = False
        verbose_name = "Task Option: Traceroute"


class TaskOptionUPnP(TaskOption):

    class Meta:
        abstract = False
        verbose_name = "Task Option: UPnP"


class Task(Base):
    method = models.TextField()
    options = models.ForeignKey("TaskOption", verbose_name=_("Task Option"))

    class Meta:
        abstract = False
        verbose_name = "Task"


class Channel(Base):
    target = models.TextField()
    certificate = models.TextField()
    timing = models.ForeignKey("Timing", verbose_name=_("Timing"))

    class Meta:
        abstract = False
        verbose_name = "Channel"


class Timing(Base):
    class Meta:
        abstract = False
        verbose_name = "Timing"

    def __str__(self):
        return "%s: %s" % (self.__class__.__name__, self.name)


class TimingPeriodic(Timing):
    start = models.DateTimeField()
    end = models.DateTimeField()
    interval = models.IntegerField()
    randomSpread = models.IntegerField(default=0)

    class Meta:
        abstract = False
        verbose_name = "Timing Periodic"


class TimingCalendar(Timing):
    start = models.DateTimeField()
    end = models.DateTimeField()
    months = models.CommaSeparatedIntegerField(blank=True, max_length=100, default="1,2,3,4,5,6,7,8,9,10,11,12")
    days_of_week = models.CommaSeparatedIntegerField(blank=True, max_length=100, default="1,2,3,4,5,6,7")
    days_of_month = models.CommaSeparatedIntegerField(blank=True, max_length=100,
                                                      default="1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17," +
                                                              "18,19,20,21,22,23,24,25,26,27,28,29,30,31")
    hours = models.CommaSeparatedIntegerField(blank=True, max_length=100,
                                              default="0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17," +
                                                      "18,19,20,21,22,23,24")
    minutes = models.CommaSeparatedIntegerField(blank=True, max_length=200,
                                                default="0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17," +
                                                        "18,19,20,21,22,23,24,25,26,27,28,29,30,31," +
                                                        "32,33,34,35,36,37,38,39,40,41,42,43,44,45," +
                                                        "46,47,48,49,50,51,52,53,54,55,56,57,58,59")
    seconds = models.CommaSeparatedIntegerField(blank=True, max_length=200,
                                                default="0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17," +
                                                        "18,19,20,21,22,23,24,25,26,27,28,29,30,31," +
                                                        "32,33,34,35,36,37,38,39,40,41,42,43,44,45," +
                                                        "46,47,48,49,50,51,52,53,54,55,56,57,58,59")
    randomSpread = models.IntegerField(default=0)

    class Meta:
        abstract = False
        verbose_name = "Timing Calender"


class TimingOneOff(Timing):
    time = models.DateTimeField()

    class Meta:
        abstract = False
        verbose_name = "Timing OneOff"


class TimingImmediate(Timing):

    class Meta:
        abstract = False
        verbose_name = "Timing Immediate"


class TimingOnDemand(Timing):

    class Meta:
        abstract = False
        verbose_name = "Timing On Demand"


class Schedule(Base):
    timing = models.ForeignKey("Timing", verbose_name=_("Timing"))
    task = models.ForeignKey("Task", verbose_name=_("Task"))
    channel = models.ForeignKey("Channel", verbose_name=_("Channel"))
    precondition = models.ForeignKey("ClientPrecondition", verbose_name=_("Client Precondition"), null=True)
    qualification = models.ForeignKey("Qualification", verbose_name=_("Qualification"), null=True, blank=True)

    excepted_results = models.TextField(blank=True)
    on_success = models.TextField(blank=True)
    on_failure = models.TextField(blank=True)

    active = models.BooleanField(default=True)
    force_remove = models.BooleanField(default=False)

    class Meta:
        abstract = False
        verbose_name = "Schedule"


class ScheduleDelivered(models.Model):
    schedule = models.ForeignKey(Schedule)
    user = models.ForeignKey(User)
    device = models.ForeignKey(Device)
    created = models.DateTimeField(auto_now_add=True)


class Qualification(Base):
    is_android = models.NullBooleanField(default=True)
    is_ios = models.NullBooleanField(default=True)
    is_windows = models.NullBooleanField(default=True)
    is_unix = models.NullBooleanField(default=True)
    is_mac = models.NullBooleanField(default=True)

    asn = models.TextField(blank=True, null=True)
    country = models.TextField(blank=True, null=True)
    group = models.IntegerField(blank=True, null=True)
    device_ids = models.CommaSeparatedIntegerField(blank=True, null=True, max_length=10000)

    class Meta:
        abstract = False
        verbose_name = "Qualification"


class ClientPrecondition(Base):
    on_wireless = models.NullBooleanField(_("Run on wireless network"), default=True)
    on_wire = models.NullBooleanField(_("Run on wire network"), default=True)
    on_cellular = models.NullBooleanField(_("Run on cellular network"), default=True)

    on_location_lat = models.FloatField(default=0.0)
    on_location_long = models.FloatField(default=0.0)
    on_location_radius = models.IntegerField(default=0)

    min_charge = models.PositiveSmallIntegerField(default=0)

    class Meta:
        abstract = False
        verbose_name = "Client Precondition"


class Configuration(Base):
    supervisor_channel = models.ForeignKey("Channel", related_name="+")
    keepalive_channel = models.ForeignKey("Channel", related_name="+")
    config_channel = models.ForeignKey("Channel", related_name="+")
    report_channel = models.ForeignKey("Channel", related_name="+")

    class Meta:
        abstract = False
        verbose_name = "Configuration"
